/*
 * Sample Program for Grafplot in Graffit < graph.c >
 *
 * XBGI  Graffit  Grafplot  ץ
 *
 * Copyright (c) 1992-94 Tokyo Denki University, Taiji Yamada
 */
#include <stdio.h>
#include <stdlib.h>
#include <graffit.h>
#include "mat.h"
#include "rk.h"

#ifndef _XBGI_
# define xbgiloop (void)getchar
#endif

/*
 * 0: vector in R2, 1: matrix in R2, 2: matrixT in R2
 * 3: matrix in R3, 4: matrixT in R3
 * 5: matrix in RN, 6: matrixT in RN
 */
int test_type = 5;

int color_kind_flag = 0;

int plot_kind_mask = PLOT_LINE;

int r3_type = R3_ROTATION;

/*
 * Lorenz
 */
#define dxdt dXdt[0]
#define dydt dXdt[1]
#define dzdt dXdt[2]
#define x    X[0]
#define y    X[1]
#define z    X[2]
void Lorenz(double t, double X[], double dXdt[])
{
  double sigma = 10, b = 8.0/3, r = 28.0;
  dxdt = -sigma*x + sigma*y;
  dydt = -x*z + r*x - y;
  dzdt = x*y - b*z;
}
#undef dxdt
#undef dydt
#undef dzdt
#undef x
#undef y
#undef z

/*
 * HyperRessler
 */
#define dxdt dXdt[0]
#define dydt dXdt[1]
#define dzdt dXdt[2]
#define dwdt dXdt[3]
#define x    X[0]
#define y    X[1]
#define z    X[2]
#define w    X[3]
void HyperRessler(double t, double X[], double dXdt[])
{
  double a = 0.25, b = 2.2, c = 0.05, d = 0.5;
  dxdt = -y - z;
  dydt = x + a*y + w;
  dzdt = b + x*z;
  dwdt = c*w - d*z;
}
#undef dxdt
#undef dydt
#undef dzdt
#undef dwdt
#undef x
#undef y
#undef z
#undef w

#define N_DATA 100

int expose( void )
{
  switch ( test_type ) {
  case 0: default:
    {
      
      int i, k;
      double data[10][N_DATA+1];
      char title[10][64];
      
      for ( k=0; k<10; k++ )
	for ( i=0; i<N_DATA+1; i++ )
	  data[k][i] = sin( 2*M_PI*(i+k*N_DATA/100)/N_DATA );
      
      set_plotgraph( "2դΥץ(vectorǡ)", 2 );
      set_plotgraph_color_kind( color_kind_flag );
      set_x_plotaxis( "X", SCALE_DEC );
      set_y_plotaxis( "Y", SCALE_DEC );
      for ( k=0; k<10; k++ ) {
	sprintf( title[k], "sin(2(t+%d)/%d)", k*N_DATA/100, N_DATA );
	set_ith_plotvector( k+1, title[k], 0,N_DATA, data[k] );
	set_ith_plotvector_kinds( k+1, plot_kind_mask, k, k, k );
      }
      setviewport_( 40, 40, 600, 440, 0 );
      setcolor( BLACK );
      setbkcolor( getmaxcolor() );
      cleardevice();
      plotgraph();
      return !0;
    }
    break;
  case 1:
    {
      int i, k;
      double **data[10];
      char title[10][64];
      
      for ( k=0; k<10; k++ ) {
	data[k] = mat(N_DATA+1,2);
	for ( i=0; i<N_DATA+1; i++ ) {
	  data[k][i][0] = 2*M_PI*i/N_DATA;
	  data[k][i][1] = sin( data[k][i][0] + 2*M_PI*(k*N_DATA/100)/N_DATA );
	}
      }
      
      set_plotgraph( "2դΥץ(matrixǡ)", 2 );
      set_plotgraph_color_kind( color_kind_flag );
      set_x_plotaxis( "X", SCALE_DEC );
      set_x_plotaxis_range( 0, 2*M_PI );
      set_y_plotaxis( "Y", SCALE_DEC );
      for ( k=0; k<10; k++ ) {
	sprintf( title[k], "y=sin(x+2%d/%d)", k*N_DATA/100, N_DATA );
	set_ith_plotmatrix( k+1, title[k], 0,N_DATA, 0,1, data[k] );
	set_ith_plotmatrix_axis( k+1, AXIS_X, 0 );
	set_ith_plotmatrix_axis( k+1, AXIS_Y, 1 );
	set_ith_plotmatrix_kinds( k+1, plot_kind_mask, k, k, k );
      }
      setviewport_( 40, 40, 600, 440, 0 );
      setcolor( BLACK );
      setbkcolor( getmaxcolor() );
      cleardevice();
      plotgraph();

      for ( k=0; k<10; k++ )
	free_mat( data[k] );
      return !0;
    }
    break;
  case 2:
    {
      int i, k;
      double **data[10];
      char title[10][64];
      
      for ( k=0; k<10; k++ ) {
	data[k] = mat(2,N_DATA+1);
	for ( i=0; i<N_DATA+1; i++ ) {
	  data[k][0][i] = 2*M_PI*i/N_DATA;
	  data[k][1][i] = sin( data[k][0][i] + 2*M_PI*(k*N_DATA/100)/N_DATA );
	}
      }
      
      set_plotgraph( "2դΥץ(matrixTǡ)", 2 );
      set_plotgraph_color_kind( color_kind_flag );
      set_x_plotaxis( "X", SCALE_DEC );
      set_x_plotaxis_range( 0, 2*M_PI );
      set_y_plotaxis( "Y", SCALE_DEC );
      for ( k=0; k<10; k++ ) {
	sprintf( title[k], "y=sin(x+2%d/%d)", k*N_DATA/100, N_DATA );
	set_ith_plotmatrixT( k+1, title[k], 0,1, 0,N_DATA, data[k] );
	set_ith_plotmatrixT_axis( k+1, AXIS_X, 0 );
	set_ith_plotmatrixT_axis( k+1, AXIS_Y, 1 );
	set_ith_plotmatrixT_kinds( k+1, plot_kind_mask, k, k, k );
      }
      setviewport_( 40, 40, 600, 440, 0 );
      setcolor( BLACK );
      setbkcolor( getmaxcolor() );
      cleardevice();
      plotgraph();

      for ( k=0; k<10; k++ )
	free_mat( data[k] );
      return !0;
    }
    break;
  case 3:
    {
      int i, n_data = 10000;
      double **data[2];
      char title[2][64];
      
      data[0] = mat(n_data+1,3);
      data[1] = mat(n_data+1,3);
      data[0][0][0] = 10; data[0][0][1] = 20; data[0][0][2] = 30;
      data[1][0][0] = 10; data[1][0][1] = 20; data[1][0][2] = 30.01;
      for ( i=0; i<n_data; i++ ) {
	rk(0.001, 3, Lorenz, 0.001*i, data[0][i], data[0][i+1] );
	rk(0.001, 3, Lorenz, 0.001*i, data[1][i], data[1][i+1] );
      }
      
      set_plotgraph( "3դΥץ(matrixǡ)", 3 );
      set_plotgraph_color_kind( color_kind_flag );
      set_plotgraph_r3kind( R3_PERSPECTIVE );
      set_x_plotaxis( "X", SCALE_DEC );
      set_y_plotaxis( "Y", SCALE_DEC );
      set_z_plotaxis( "Z", SCALE_DEC );

      sprintf( title[0], "(Z(0)=%g))", data[0][0][2] );
      set_ith_plotmatrix( 1, title[0], 0,n_data, 0,2, data[0] );
      set_ith_plotmatrix_kinds( 1, plot_kind_mask, 0, 0, 0 );

      sprintf( title[1], "(Z(0)=%g))", data[1][0][2] );
      set_ith_plotmatrix( 2, title[1], 0,n_data, 0,2, data[1] );
      set_ith_plotmatrix_kinds( 2, plot_kind_mask, 1, 1, 1 );

      setviewport_( 40, 40, 600, 440, 0 );
      setwindow_( -1.0/1.5, 1.0/1.5, 1.0/1.5, -1.0/1.5 );
      setrotation_( RAD(-20), RAD(30), RAD(0) );
      setdistance_( 0.5 );
      setcolor( BLACK );
      setbkcolor( getmaxcolor() );
      cleardevice();
      plotgraph();

      free_mat( data[0] );
      free_mat( data[1] );
      return !0;
    }
    break;
  case 4:
    {
      int i, n_data = 10000;
      double **data[2], *v0 = vec(3), *v1 = vec(3);
      char title[2][64];
      
      data[0] = mat(3,n_data+1);
      data[1] = mat(3,n_data+1);
      data[0][0][0] = 10; data[0][1][0] = 20; data[0][2][0] = 30;
      data[1][0][0] = 10; data[1][1][0] = 20; data[1][2][0] = 30.01;
      for ( i=0; i<n_data; i++ ) {
	v0[0] = data[0][0][i];
	v0[1] = data[0][1][i];
	v0[2] = data[0][2][i];
	rk(0.001, 3, Lorenz, 0.001*i, v0, v1 );
	data[0][0][i+1] = v1[0];
	data[0][1][i+1] = v1[1];
	data[0][2][i+1] = v1[2];
	v0[0] = data[1][0][i];
	v0[1] = data[1][1][i];
	v0[2] = data[1][2][i];
	rk(0.001, 3, Lorenz, 0.001*i, v0, v1 );
	data[1][0][i+1] = v1[0];
	data[1][1][i+1] = v1[1];
	data[1][2][i+1] = v1[2];
      }
      
      set_plotgraph( "3դΥץ(matrixTǡ)", 3 );
      set_plotgraph_color_kind( color_kind_flag );
      set_plotgraph_r3kind( R3_PERSPECTIVE );
      set_x_plotaxis( "X", SCALE_DEC );
      set_y_plotaxis( "Y", SCALE_DEC );
      set_z_plotaxis( "Z", SCALE_DEC );

      sprintf( title[0], "(Z(0)=%g))", data[0][2][0] );
      set_ith_plotmatrixT( 1, title[0], 0,2, 0,n_data, data[0] );
      set_ith_plotmatrixT_kinds( 1, plot_kind_mask, 0, 0, 0 );

      sprintf( title[1], "(Z(0)=%g))", data[1][2][0] );
      set_ith_plotmatrixT( 2, title[1], 0,2, 0,n_data, data[1] );
      set_ith_plotmatrixT_kinds( 2, plot_kind_mask, 1, 1, 1 );

      setviewport_( 40, 40, 600, 440, 0 );
      setwindow_( -1.0/1.5, 1.0/1.5, 1.0/1.5, -1.0/1.5 );
      setrotation_( RAD(-20), RAD(30), RAD(0) );
      setdistance_( 0.5 );
      setcolor( BLACK );
      setbkcolor( getmaxcolor() );
      cleardevice();
      plotgraph();

      free_vec( v0 );
      free_vec( v1 );
      free_mat( data[0] );
      free_mat( data[1] );
      return !0;
    }
    break;
  case 5:
    {
      int i, n_data = 10000;
      double **data[2], *v0 = vec(4), *v1 = vec(4);
      char title[2][64];
      
      data[0] = mat(n_data+1,4);
      data[1] = mat(n_data+1,4);
      data[0][0][0]=-19; data[0][0][1]=0; data[0][0][2]=0; data[0][0][3]=15;
      data[1][0][0]=-19; data[1][0][1]=0; data[1][0][2]=0; data[1][0][3]=14.9;
      for ( i=0; i<n_data; i++ ) {
	rk(0.01, 4, HyperRessler, 0.01*i, data[0][i], data[0][i+1] );
	rk(0.01, 4, HyperRessler, 0.01*i, data[1][i], data[1][i+1] );
      }
      
      set_plotgraph( "4դΥץ(matrixǡ)", 4 );
      set_plotgraph_color_kind( color_kind_flag );
      set_plotgraph_r3kind( R3_PERSPECTIVE );
      set_x_plotaxis( "X", SCALE_DEC );
      set_y_plotaxis( "Y", SCALE_DEC );
      set_z_plotaxis( "Z", SCALE_DEC );
      set_w_plotaxis( "W", SCALE_DEC );

      sprintf( title[0], "쥹顼(W(0)=%g))", data[0][0][3] );
      set_ith_plotmatrix( 1, title[0], 0,n_data, 0,3, data[0] );
      set_ith_plotmatrix_kinds( 1, plot_kind_mask, 0, 0, 0 );

      sprintf( title[1], "쥹顼(W(0)=%g))", data[1][0][3] );
      set_ith_plotmatrix( 2, title[1], 0,n_data, 0,3, data[1] );
      set_ith_plotmatrix_kinds( 2, plot_kind_mask, 1, 1, 1 );

      setviewport_( 40, 40, 600, 440, 0 );
      setwindow_( -1.0/1.5, 1.0/1.5, 1.0/1.5, -1.0/1.5 );
      setrotationport_( -0.5, -0.5, -0.5, 0.5, 0.5, 0.5 );
      setrotation_( RAD(-20), RAD(30), RAD(0) );
      setdistance_( 0.5 );
      setcolor( BLACK );
      setbkcolor( getmaxcolor() );
      cleardevice();
      plotgraph();

      free_vec( v0 );
      free_vec( v1 );
      free_mat( data[0] );
      free_mat( data[1] );
      return !0;
    }
    break;
  case 6:
    {
      int i, n_data = 10000;
      double **data[2], *v0 = vec(4), *v1 = vec(4);
      char title[2][64];
      
      data[0] = mat(4,n_data+1);
      data[1] = mat(4,n_data+1);
      data[0][0][0]=-19; data[0][1][0]=0; data[0][2][0]=0; data[0][3][0]=15;
      data[1][0][0]=-19; data[1][1][0]=0; data[1][2][0]=0; data[1][3][0]=14.9;
      for ( i=0; i<n_data; i++ ) {
	v0[0] = data[0][0][i];
	v0[1] = data[0][1][i];
	v0[2] = data[0][2][i];
	v0[3] = data[0][3][i];
	rk(0.01, 4, HyperRessler, 0.01*i, v0, v1 );
	data[0][0][i+1] = v1[0];
	data[0][1][i+1] = v1[1];
	data[0][2][i+1] = v1[2];
	data[0][3][i+1] = v1[3];
	v0[0] = data[1][0][i];
	v0[1] = data[1][1][i];
	v0[2] = data[1][2][i];
	v0[3] = data[1][3][i];
	rk(0.01, 4, HyperRessler, 0.01*i, v0, v1 );
	data[1][0][i+1] = v1[0];
	data[1][1][i+1] = v1[1];
	data[1][2][i+1] = v1[2];
	data[1][3][i+1] = v1[3];
      }
      
      set_plotgraph( "4դΥץ(matrixTǡ)", 4 );
      set_plotgraph_color_kind( color_kind_flag );
      set_plotgraph_r3kind( R3_PERSPECTIVE );
      set_x_plotaxis( "X", SCALE_DEC );
      set_y_plotaxis( "Y", SCALE_DEC );
      set_z_plotaxis( "Z", SCALE_DEC );
      set_w_plotaxis( "W", SCALE_DEC );

      sprintf( title[0], "쥹顼(W(0)=%g))", data[0][3][0] );
      set_ith_plotmatrixT( 1, title[0], 0,3, 0,n_data, data[0] );
      set_ith_plotmatrixT_kinds( 1, plot_kind_mask, 0, 0, 0 );

      sprintf( title[1], "쥹顼(W(0)=%g))", data[1][3][0] );
      set_ith_plotmatrixT( 2, title[1], 0,3, 0,n_data, data[1] );
      set_ith_plotmatrixT_kinds( 2, plot_kind_mask, 1, 1, 1 );

      setviewport_( 40, 40, 600, 440, 0 );
      setwindow_( -1.0/1.5, 1.0/1.5, 1.0/1.5, -1.0/1.5 );
      setrotationport_( -0.5, -0.5, -0.5, 0.5, 0.5, 0.5 );
      setrotation_( RAD(-20), RAD(30), RAD(0) );
      setdistance_( 0.5 );
      setcolor( BLACK );
      setbkcolor( getmaxcolor() );
      cleardevice();
      plotgraph();

      free_vec( v0 );
      free_vec( v1 );
      free_mat( data[0] );
      free_mat( data[1] );
      return !0;
    }
    break;
  }
}
int key_press( int key )
{
  switch( key ) {
  case '1': case '2': case '3': case '4': case '5': case '6':
    test_type = key - '0';
    expose();
    break;
  case 'D':
    switch ( plot_kind_mask ) {
    case PLOT_LINE: plot_kind_mask = PLOT_DOT; break;
    case PLOT_DOT: plot_kind_mask = PLOT_POINT; break;
    case PLOT_POINT: plot_kind_mask = PLOT_LINE|PLOT_POINT; break;
    case PLOT_LINE|PLOT_POINT: plot_kind_mask = PLOT_LINE; break;
    }
    expose();
    break;
  case 'T':
    r3_type = (r3_type==R3_ROTATION)?R3_PERSPECTIVE:R3_ROTATION;
    expose();
    break;
  case 'C':
    color_kind_flag = !color_kind_flag;
    expose();
   break;
  case '*':
    {
      int i, j;
      
      for ( i=0; i<4; i++ )
	for ( j=0; j<4; j++ )
	  if ( i != j )
	    hyperrotate_( i, j, RAD(5) );
    }
    expose();
    break;
  }
  return !0;
}
int main( void )
{
  int gd = DETECT, gm;

  initgraph( &gd, &gm, "-geometry 640x480 -title graph" );
  expose();
  set_key_press_handler( key_press );
  set_expose_handler( expose );
  xbgiloop();
  closegraph();
  return 0;
}
